/*
 * moveto
 *
 * based off autowarp.c and playercmd.c
 *
 * 29/06/04 - Started by Smong (to prove writing modules is easy).
 *
 * 01/07/04 - Changed SPEC to SHIP_SPEC.
 *
 * 03/01/05 - Updated for asss 1.3.2.
 *
 */

#include <stdlib.h>
#include "asss.h"

local void Cmoveto(const char *command, const char *params, Player *p,
	const Target *target);

local void do_checksum(struct S2CWeapons *pkt);
local void move_player(Player *p, int x, int y, ticks_t delta);

local Imodman *mm;
local Inet *net;
local Icmdman *cmd;
local Iplayerdata *pd;

local void do_checksum(struct S2CWeapons *pkt)
{
	int i;
	u8 ck = 0;
	pkt->checksum = 0;
	for (i = 0; i < sizeof(struct S2CWeapons) - sizeof(struct ExtraPosData); i++)
		ck ^= ((unsigned char*)pkt)[i];
	pkt->checksum = ck;
}

local void move_player(Player *p, int x, int y, ticks_t delta)
{
	struct S2CWeapons wpn = {
		S2C_WEAPON, p->position.rotation, (delta + 10000) & 0xFFFF,
		x, p->position.yspeed, p->pid, p->position.xspeed, 0,
		p->position.status, 0, y, p->position.bounty
	};

	do_checksum(&wpn);
	net->SendToOne(p, (byte*)&wpn, sizeof(struct S2CWeapons) -
		sizeof(struct ExtraPosData), NET_PRI_P4);
}

local helptext_t moveto_help =
"Targets: player, freq, or arena\n"
"Args: <x coord> <y coord>\n"
"Moves target player to coordinate x,y.\n";

local void Cmoveto(const char *command, const char *params, Player *p,
	const Target *target)
{
	char *next;
	int x, y;
	Link *l;
	LinkedList set = LL_INITIALIZER;
	ticks_t now;

	x = strtol(params, &next, 0);
	if (next == params) return;
	while (*next == ',' || *next == ' ') next++;
	y = strtol(next, NULL, 0);
	if (x == 0 || y == 0) return;

	x = x << 4;
	y = y << 4;	
	pd->TargetToSet(target, &set);
	now = current_ticks();

	for (l = LLGetHead(&set); l; l = l->next)
	{
		p = l->data;
		if (p->p_ship == SHIP_SPEC || !IS_STANDARD(p)) continue;

		move_player(p, x, y, now);
	}
}

EXPORT const char info_moveto[] = "v1.1 smong <soinsg@hotmail.com>";

EXPORT int MM_moveto(int action, Imodman *mm_, Arena *arena)
{
	if (action == MM_LOAD)
	{
		mm = mm_;
		net = mm->GetInterface(I_NET, ALLARENAS);
		cmd = mm->GetInterface(I_CMDMAN, ALLARENAS);
		pd = mm->GetInterface(I_PLAYERDATA, ALLARENAS);
		if (!net || !cmd || !pd) return MM_FAIL;
		cmd->AddCommand("moveto", Cmoveto, ALLARENAS, moveto_help);
		return MM_OK;
	}
	else if (action == MM_UNLOAD)
	{
		cmd->RemoveCommand("moveto", Cmoveto, ALLARENAS);
		mm->ReleaseInterface(net);
		mm->ReleaseInterface(cmd);
		mm->ReleaseInterface(pd);
		return MM_OK;
	}

	return MM_FAIL;
}
